﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LinqToObject
{
    class Osoba
    {
        public int Id;
        public string Imię, Nazwisko;
        public int NumerTelefonu;
        public DateTime DataUrodzenia;

        public override string ToString()
        {
            return 
                Id.ToString() + ". " + 
                Imię + " " + Nazwisko + 
                " (" + DataUrodzenia.ToLongDateString() + "), nr tel. " + NumerTelefonu;
        }
    }

    class Program
    {
        private static List<Osoba> pobierzOsoby()
        {
            List<Osoba> listaOsób = new List<Osoba>()
            {
                new Osoba() { Id = 1, Imię = "Krzysztof", Nazwisko = "Kruglak", DataUrodzenia = new DateTime(1998, 6, 5), NumerTelefonu = 123456890 },
                new Osoba() { Id = 2, Imię = "Korneliusz", Nazwisko = "Łożyński", DataUrodzenia = new DateTime(1999, 5, 27), NumerTelefonu = 34985349 },
                new Osoba() { Id = 3, Imię = "Kacper", Nazwisko = "Kowalski", DataUrodzenia = new DateTime(1999, 10, 17), NumerTelefonu = 123456890 },
                new Osoba() { Id = 4, Imię = "Bartosz", Nazwisko = "Grabowski", DataUrodzenia = new DateTime(1998, 4, 4), NumerTelefonu = 34238786 },
                new Osoba() { Id = 5, Imię = "Patryk", Nazwisko = "Barański", DataUrodzenia = new DateTime(1999, 3, 22), NumerTelefonu = 23423894 }
            };
            return listaOsób;
        }

        private static int obliczWiek(DateTime dataUrodzenia)
        {
            //TimeSpan ts = DateTime.Now - dataUrodzenia;
            int wiek = DateTime.Now.Year - dataUrodzenia.Year;
            return wiek;
        }

        static void Main(string[] args)
        {
            List<Osoba> listaOsób1 = pobierzOsoby();

            var zapytanie = from osoba in listaOsób1
                            //where osoba.Imię.StartsWith("K")
                            where obliczWiek(osoba.DataUrodzenia) > 20
                            //orderby osoba.DataUrodzenia descending
                            orderby obliczWiek(osoba.DataUrodzenia) descending
                            //select new { osoba.Imię, osoba.Nazwisko, osoba.DataUrodzenia };
                            select osoba;

            Console.WriteLine("Wynik zapytania:");
            foreach(var element in zapytanie)
            {
                Console.WriteLine(element.ToString());
            }

            //zapytanie.ElementAt(1).Imię = "Nowe imię";

            Console.WriteLine(listaOsób1.Average(o => obliczWiek(o.DataUrodzenia)));
            Console.WriteLine(zapytanie.Average(o => obliczWiek(o.DataUrodzenia)));
            Console.WriteLine();

            Console.WriteLine(listaOsób1.Min(o => obliczWiek(o.DataUrodzenia)));
            Console.WriteLine(zapytanie.Min(o => obliczWiek(o.DataUrodzenia)));
            Console.WriteLine();

            Console.WriteLine(listaOsób1.Max(o => obliczWiek(o.DataUrodzenia)));
            Console.WriteLine(zapytanie.Max(o => obliczWiek(o.DataUrodzenia)));
            Console.WriteLine();

            Console.WriteLine(listaOsób1.First());
            Console.WriteLine(listaOsób1.First(o => o.Imię[0] != 'K'));

            Console.WriteLine(listaOsób1.All(o => o.Imię.StartsWith("K")));
            Console.WriteLine(listaOsób1.Any(o => o.Imię.StartsWith("K")));

            var roczniki = from Osoba osoba in listaOsób1
                           group osoba by osoba.DataUrodzenia.Year into grupa
                           select grupa;
            foreach(var grupa in roczniki)
            {
                Console.WriteLine("Rocznik " + grupa.Key);
                foreach(Osoba osoba in grupa)
                {
                    Console.WriteLine(osoba.ToString());
                }
                Console.WriteLine();
            }

            var listaOsóbZImieniemNaLiteręK = from Osoba osoba in listaOsób1
                                              where osoba.Imię.StartsWith("K")
                                              select osoba;
            var listaOsóbZRocznika1998 = from Osoba osoba in listaOsób1
                                         where osoba.DataUrodzenia.Year == 1998
                                         select osoba;

            //var listaOsóbZImieniemNaLiteręKRocznik1998 = listaOsóbZImieniemNaLiteręK.Concat(listaOsóbZRocznika1998).Distinct();
            var listaOsóbZImieniemNaLiteręKRocznik1998 = listaOsóbZImieniemNaLiteręK.Union(listaOsóbZRocznika1998);            
            foreach (var element in listaOsóbZImieniemNaLiteręKRocznik1998)
            {
                Console.WriteLine(element.ToString());
            }
            Console.WriteLine();

            var listaOsóbZImieniemNaLiteręKNieZRocznik1998 = listaOsóbZImieniemNaLiteręK.Except(listaOsóbZRocznika1998);
            foreach (var element in listaOsóbZImieniemNaLiteręKNieZRocznik1998)
            {
                Console.WriteLine(element.ToString());
            }
            Console.WriteLine();

            var częśćWspólna = listaOsóbZImieniemNaLiteręK.Intersect(listaOsóbZRocznika1998);
            foreach (var element in częśćWspólna)
            {
                Console.WriteLine(element.ToString());
            }
            Console.WriteLine();

            var personalia = from Osoba osoba in listaOsób1
                             select new { osoba.Id, osoba.Imię, osoba.Nazwisko };

            var datyUrodzenia = from Osoba osoba in listaOsób1
                                select new { osoba.Id, osoba.DataUrodzenia };

            var listaOsób2 = from imięINazwisko in personalia
                             join dataUrodzenia in datyUrodzenia
                             on imięINazwisko.Id equals dataUrodzenia.Id
                             select new { imięINazwisko.Id, imięINazwisko.Imię, imięINazwisko.Nazwisko, dataUrodzenia.DataUrodzenia };
            foreach (var element in listaOsób2)
            {
                Console.WriteLine(element.ToString());
            }
            Console.WriteLine();
        }
    }
}
